@@ -24,8 +24,14 @@ var LibraryBrowser = {
                      'Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };\n' +
                      'Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };\n' +
                      'Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };\n' +
-                     'Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }\n' +
-                     'Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }',
+                     'Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() };\n' +
+                     'Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) };\n' +
+#if MAIN_MODULE
+                     'var preloadedWasm = {};\n' +
+#endif
+                     'var preloadedImages = {};\n' +
+                     'var preloadedAudios = {};\n',
+
   $Browser: {
     mainLoop: {
       running: false,
@@ -110,7 +116,7 @@ var LibraryBrowser = {
       }
       Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? out("warning: no BlobBuilder") : null));
       Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
-      if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
+      if (!Module.noImageDecoding && typeof Browser.URLObject == 'undefined') {
         out("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
         Module.noImageDecoding = true;
       }
@@ -150,18 +156,18 @@ var LibraryBrowser = {
         assert(typeof url == 'string', 'createObjectURL must return a url as a string');
 #endif
         var img = new Image();
-        img.onload = function img_onload() {
+        img.onload = () => {
           assert(img.complete, 'Image ' + name + ' could not be decoded');
-          var canvas = document.createElement('canvas');
+          var canvas = /** @type {!HTMLCanvasElement} */ (document.createElement('canvas'));
           canvas.width = img.width;
           canvas.height = img.height;
           var ctx = canvas.getContext('2d');
           ctx.drawImage(img, 0, 0);
-          Module["preloadedImages"][name] = canvas;
+          preloadedImages[name] = canvas;
           Browser.URLObject.revokeObjectURL(url);
           if (onload) onload(byteArray);
         };
-        img.onerror = function img_onerror(event) {
+        img.onerror = (event) => {
           out('Image ' + url + ' could not be decoded');
           if (onerror) onerror();
         };
@@ -178,13 +184,13 @@ var LibraryBrowser = {
         function finish(audio) {
           if (done) return;
           done = true;
-          Module["preloadedAudios"][name] = audio;
+          preloadedAudios[name] = audio;
           if (onload) onload(byteArray);
         }
         function fail() {
           if (done) return;
           done = true;
-          Module["preloadedAudios"][name] = new Audio(); // empty shim
+          preloadedAudios[name] = new Audio(); // empty shim
           if (onerror) onerror();
         }
         if (Browser.hasBlobConstructor) {
@@ -252,12 +258,12 @@ var LibraryBrowser = {
           // loadWebAssemblyModule can not load modules out-of-order, so rather
           // than just running the promises in parallel, this makes a chain of
           // promises to run in series.
-          this['asyncWasmLoadPromise'] = this['asyncWasmLoadPromise'].then(
+          wasmPlugin['asyncWasmLoadPromise'] = wasmPlugin['asyncWasmLoadPromise'].then(
             function() {
               return loadWebAssemblyModule(byteArray, {loadAsync: true, nodelete: true});
             }).then(
               function(module) {
-                Module['preloadedWasm'][name] = module;
+                preloadedWasm[name] = module;
                 onload();
               },
               function(err) {
@@ -310,7 +316,24 @@ var LibraryBrowser = {
       }
     },
 
-    createContext: function(canvas, useWebGL, setInModule, webGLContextAttributes) {
+    // Tries to handle an input byteArray using preload plugins. Returns true if
+    // it was handled.
+    handledByPreloadPlugin: function(byteArray, fullname, finish, onerror) {
+      // Ensure plugins are ready.
+      Browser.init();
+
+      var handled = false;
+      Module['preloadPlugins'].forEach(function(plugin) {
+        if (handled) return;
+        if (plugin['canHandle'](fullname)) {
+          plugin['handle'](byteArray, fullname, finish, onerror);
+          handled = true;
+        }
+      });
+      return handled;
+    },
+
+    createContext: function(/** @type {HTMLCanvasElement} */ canvas, useWebGL, setInModule, webGLContextAttributes) {
       if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
 
       var ctx;
@@ -322,12 +345,10 @@ var LibraryBrowser = {
           alpha: false,
 #if MIN_WEBGL_VERSION >= 2
           majorVersion: 2,
-#else
-#if MAX_WEBGL_VERSION >= 2 // library_browser.js defaults: use the WebGL version chosen at compile time (unless overridden below)
-          majorVersion: (typeof WebGL2RenderingContext !== 'undefined') ? 2 : 1,
+#elif MAX_WEBGL_VERSION >= 2 // library_browser.js defaults: use the WebGL version chosen at compile time (unless overridden below)
+          majorVersion: (typeof WebGL2RenderingContext != 'undefined') ? 2 : 1,
 #else
           majorVersion: 1,
-#endif
 #endif
         };
 
@@ -340,7 +361,7 @@ var LibraryBrowser = {
         // This check of existence of GL is here to satisfy Closure compiler, which yells if variable GL is referenced below but GL object is not
         // actually compiled in because application is not doing any GL operations. TODO: Ideally if GL is not being used, this function
         // Browser.createContext() should not even be emitted.
-        if (typeof GL !== 'undefined') {
+        if (typeof GL != 'undefined') {
           contextHandle = GL.createContext(canvas, contextAttributes);
           if (contextHandle) {
             ctx = GL.getContext(contextHandle).GLctx;
@@ -353,7 +374,7 @@ var LibraryBrowser = {
       if (!ctx) return null;
 
       if (setInModule) {
-        if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
+        if (!useWebGL) assert(typeof GLctx == 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
 
         Module.ctx = ctx;
         if (useWebGL) GL.makeContextCurrent(contextHandle);
@@ -372,8 +393,8 @@ var LibraryBrowser = {
     requestFullscreen: function(lockPointer, resizeCanvas) {
       Browser.lockPointer = lockPointer;
       Browser.resizeCanvas = resizeCanvas;
-      if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
-      if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
+      if (typeof Browser.lockPointer == 'undefined') Browser.lockPointer = true;
+      if (typeof Browser.resizeCanvas == 'undefined') Browser.resizeCanvas = false;
 
       var canvas = Module['canvas'];
       function fullscreenChange() {
@@ -469,13 +490,13 @@ var LibraryBrowser = {
     },
 
     requestAnimationFrame: function(func) {
-      if (typeof requestAnimationFrame === 'function') {
+      if (typeof requestAnimationFrame == 'function') {
         requestAnimationFrame(func);
         return;
       }
       var RAF = Browser.fakeRequestAnimationFrame;
 #if LEGACY_VM_SUPPORT
-      if (typeof window !== 'undefined') {
+      if (typeof window != 'undefined') {
         RAF = window['requestAnimationFrame'] ||
               window['mozRequestAnimationFrame'] ||
               window['webkitRequestAnimationFrame'] ||
@@ -622,12 +643,12 @@ var LibraryBrowser = {
         // Neither .scrollX or .pageXOffset are defined in a spec, but
         // we prefer .scrollX because it is currently in a spec draft.
         // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
-        var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
-        var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
+        var scrollX = ((typeof window.scrollX != 'undefined') ? window.scrollX : window.pageXOffset);
+        var scrollY = ((typeof window.scrollY != 'undefined') ? window.scrollY : window.pageYOffset);
 #if ASSERTIONS
         // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
         // and we have no viable fallback.
-        assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
+        assert((typeof scrollX != 'undefined') && (typeof scrollY != 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
 #endif
 
         if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
@@ -694,7 +715,7 @@ var LibraryBrowser = {
       if (typeof SDL != "undefined") {
         var flags = {{{ makeGetValue('SDL.screen', '0', 'i32', 0, 1) }}};
         flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
-        {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}}
+        {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
       Browser.updateCanvasDimensions(Module['canvas']);
       Browser.updateResizeListeners();
@@ -705,7 +726,7 @@ var LibraryBrowser = {
       if (typeof SDL != "undefined") {
         var flags = {{{ makeGetValue('SDL.screen', '0', 'i32', 0, 1) }}};
         flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
-        {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}}
+        {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
       Browser.updateCanvasDimensions(Module['canvas']);
       Browser.updateResizeListeners();
@@ -758,36 +779,6 @@ var LibraryBrowser = {
     },
   },
 
-  $funcWrappers: {},
-
-  $getFuncWrapper__deps: ['$funcWrappers', '$dynCall'],
-  $getFuncWrapper: function(func, sig) {
-    if (!func) return; // on null pointer, return undefined
-    assert(sig);
-    if (!funcWrappers[sig]) {
-      funcWrappers[sig] = {};
-    }
-    var sigCache = funcWrappers[sig];
-    if (!sigCache[func]) {
-      // optimize away arguments usage in common cases
-      if (sig.length === 1) {
-        sigCache[func] = function dynCall_wrapper() {
-          return dynCall(sig, func);
-        };
-      } else if (sig.length === 2) {
-        sigCache[func] = function dynCall_wrapper(arg) {
-          return dynCall(sig, func, [arg]);
-        };
-      } else {
-        // general case
-        sigCache[func] = function dynCall_wrapper() {
-          return dynCall(sig, func, Array.prototype.slice.call(arguments));
-        };
-      }
-    }
-    return sigCache[func];
-  },
-
   emscripten_run_preload_plugins__deps: ['$PATH',
 #if !MINIMAL_RUNTIME
     '$runtimeKeepalivePush', '$runtimeKeepalivePop',
@@ -927,11 +918,11 @@ var LibraryBrowser = {
       };
       Browser.mainLoop.method = 'rAF';
     } else if (mode == 2 /*EM_TIMING_SETIMMEDIATE*/) {
-      if (typeof setImmediate === 'undefined') {
+      if (typeof setImmediate == 'undefined') {
         // Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
         var setImmediates = [];
         var emscriptenMainLoopMessageId = 'setimmediate';
-        var Browser_setImmediate_messageHandler = function(event) {
+        var Browser_setImmediate_messageHandler = function(/** @type {Event} */ event) {
           // When called in current thread or Worker, the main loop ID is structured slightly different to accommodate for --proxy-to-worker runtime listening to Worker events,
           // so check for both cases.
           if (event.data === emscriptenMainLoopMessageId || event.data.target === emscriptenMainLoopMessageId) {
@@ -958,6 +949,7 @@ var LibraryBrowser = {
   },
 
   emscripten_set_main_loop__deps: ['$setMainLoop'],
+  emscripten_set_main_loop__sig: 'viii',
   emscripten_set_main_loop: function(func, fps, simulateInfiniteLoop) {
     var browserIterationFunc = {{{ makeDynCall('v', 'func') }}};
     setMainLoop(browserIterationFunc, fps, simulateInfiniteLoop);
@@ -974,6 +966,11 @@ var LibraryBrowser = {
     '$maybeExit',
 #endif
   ],
+  $setMainLoop__docs: `
+  /**
+   * @param {number=} arg
+   * @param {boolean=} noSetTiming
+   */`,
   $setMainLoop: function(browserIterationFunc, fps, simulateInfiniteLoop, arg, noSetTiming) {
     assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
 
@@ -1063,7 +1060,7 @@ var LibraryBrowser = {
 #if USE_PTHREADS && OFFSCREEN_FRAMEBUFFER && GL_SUPPORT_EXPLICIT_SWAP_CONTROL
       // If the current GL context is a proxied regular WebGL context, and was initialized with implicit swap mode on the main thread, and we are on the parent thread,
       // perform the swap on behalf of the user.
-      if (typeof GL !== 'undefined' && GL.currentContext && GL.currentContextIsProxied) {
+      if (typeof GL != 'undefined' && GL.currentContext && GL.currentContextIsProxied) {
         var explicitSwapControl = {{{ makeGetValue('GL.currentContext', 0, 'i32') }}};
         if (!explicitSwapControl) _emscripten_webgl_commit_frame();
       }
@@ -1071,7 +1068,7 @@ var LibraryBrowser = {
 
 #if OFFSCREENCANVAS_SUPPORT
       // If the current GL context is an OffscreenCanvas, but it was initialized with implicit swap mode, perform the swap on behalf of the user.
-      if (typeof GL !== 'undefined' && GL.currentContext && !GL.currentContextIsProxied && !GL.currentContext.attributes.explicitSwapControl && GL.currentContext.GLctx.commit) {
+      if (typeof GL != 'undefined' && GL.currentContext && !GL.currentContextIsProxied && !GL.currentContext.attributes.explicitSwapControl && GL.currentContext.GLctx.commit) {
         GL.currentContext.GLctx.commit();
       }
 #endif
@@ -1096,7 +1093,7 @@ var LibraryBrowser = {
       // to queue the newest produced audio samples.
       // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
       //       do not need to be hardcoded into this function, but can be more generic.
-      if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
+      if (typeof SDL == 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
 
       Browser.mainLoop.scheduler();
     }
@@ -1193,10 +1190,7 @@ var LibraryBrowser = {
       _emscripten_get_window_title.buffer = _malloc(buflen);
     }
 
-    writeAsciiToMemory(
-      document.title.slice(0, buflen - 1),
-      _emscripten_get_window_title.buffer
-    );
+    stringToUTF8(document.title, _emscripten_get_window_title.buffer, buflen);
 
     return _emscripten_get_window_title.buffer;
   },
@@ -1204,7 +1198,7 @@ var LibraryBrowser = {
   emscripten_set_window_title__proxy: 'sync',
   emscripten_set_window_title__sig: 'vi',
   emscripten_set_window_title: function(title) {
-    setWindowTitle(AsciiToString(title));
+    setWindowTitle(UTF8ToString(title));
   },
 
   emscripten_get_screen_size__proxy: 'sync',
@@ -1381,7 +1375,7 @@ var LibraryBrowser = {
 
     path = PATH_FS.resolve(path);
 
-    var canvas = Module["preloadedImages"][path];
+    var canvas = /** @type {HTMLCanvasElement} */(preloadedImages[path]);
     if (canvas) {
       var ctx = canvas.getContext("2d");
       var image = ctx.getImageData(0, 0, canvas.width, canvas.height);
